/*
 *  Copyright (C) 2002  Ricardo Fernndez Pascual
 *  Copyright (C) 2004  Crispin Flowerday
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <libgnome/gnome-i18n.h>

#include "egg-menu-merge.h"
#include "egg-action-group.h"

#include "galeon-window.h"
#include "galeon-tab.h"
#include "galeon-radio-action.h"
#include "galeon-action-group.h"
#include "galeon-css-menu.h"

#include "gul-gobject-misc.h"
#include "galeon-marshal.h"
#include "galeon-debug.h"
#include "gul-string.h"
#include "gul-gui.h"

/**
 * Private data
 */
struct _GaleonCssMenuPrivate
{
	GaleonWindow *window;
	EggMenuMerge *merge;
	GaleonEmbed *embed;

	EggActionGroup *item_action_group;
	EggActionGroup *menu_action_group;
	guint merge_id;

	guint rebuild_timeout;
	gboolean block_signals;
};

enum
{
	PROP_0,
	PROP_WINDOW
};

static void galeon_css_menu_rebuild (GaleonCssMenu *menu);

static gpointer g_object_class;

#define STYLESHEET_KEY                  "galeon-stylesheet"
#define REBUILD_TIMEOUT 700

static void
activate_stylesheet_cb (EggAction *action, GaleonCssMenu *menu)
{
	GaleonCssMenuPrivate *p = menu->priv;
	EmbedStyleSheet *style;

	if (menu->priv->block_signals) return;

	g_return_if_fail (IS_GALEON_EMBED (p->embed));

	style = g_object_get_data (G_OBJECT (action), STYLESHEET_KEY);

	galeon_embed_set_stylesheet (p->embed, style);

	/* Hack, re-build the menu here, so the radio buttons appear correct */
	galeon_css_menu_rebuild (menu);
}

static EggAction *
create_stylesheet_action (GaleonCssMenu *menu, EmbedStyleSheet *style)
{
	EggAction  *action;
	gchar *name, *tooltip, *label;

	label = gul_string_double_underscores (style->name);

	switch (style->type)
	{
	case STYLESHEET_NONE:
		tooltip = g_strdup (_("Render the page without using a style"));
		break;
	case STYLESHEET_BASIC:
		tooltip = g_strdup (_("Render the page using the default style"));
		break;
	default:
		tooltip = g_strdup_printf (_("Render the page using the \"%s\" style"), 
					   style->name);
		break;
	}

	name = g_strdup_printf ("GaleonCSS%sAction", style->name);
	action = g_object_new (GALEON_TYPE_RADIO_ACTION,
			       "name", name,
			       "label", label,
			       "tooltip", tooltip,
			       "window", menu->priv->window,
			       NULL);

	g_object_set_data_full (G_OBJECT(action), STYLESHEET_KEY, style,
				(GDestroyNotify)galeon_embed_stylesheet_free);

	g_signal_connect_object (action, "activate",
				 G_CALLBACK (activate_stylesheet_cb),
				 menu, 0);

	egg_action_group_add_action (menu->priv->item_action_group, action);
	g_object_unref (action);

	g_free (name);
	g_free (label);
	g_free (tooltip);
	return action;
}

static void
galeon_css_menu_rebuild (GaleonCssMenu *menu)
{
	GaleonCssMenuPrivate *p = menu->priv;
	GString *xml;
	GList *stylesheets = 0, *l;
	EggAction *action;
	gresult result;
	EmbedStyleSheet *current = NULL;
	GSList *radio_group = NULL;

	if (p->merge_id > 0)
	{
		egg_menu_merge_remove_ui (p->merge, p->merge_id);
		egg_menu_merge_ensure_update (p->merge);
		p->merge_id = 0;
	}

	if (p->item_action_group)
	{
		egg_menu_merge_remove_action_group (p->merge, p->item_action_group);
		g_object_unref (p->item_action_group);
		p->item_action_group = 0;
	}

	LOG ("Rebuilding stylesheet menu");

	result = galeon_embed_get_stylesheets (p->embed, &stylesheets);
	if (result != G_OK) stylesheets = NULL;

	result = galeon_embed_get_selected_stylesheet (p->embed, &current);
	if (result != G_OK) current = NULL;

	p->block_signals = TRUE;

	/* Create the new action group */
	p->item_action_group = 
		egg_action_group_new ("StylesheetMenuDynamicActions");
	egg_menu_merge_insert_action_group (p->merge, p->item_action_group, 0);

	xml = g_string_new (NULL);
	g_string_append (xml, 
			 "<Root><menu><submenu name=\"View\">\n"
			 "<placeholder name=\"ViewStylesMenuPlaceholder\">\n"
			 "<submenu name=\"StyleMenuActionItem\" "
			 "verb=\"StyleMenuAction\">");
	
	action = egg_action_group_get_action (p->menu_action_group,
					      "StyleMenuAction");
	g_object_set (G_OBJECT (action), "sensitive", (stylesheets != NULL), NULL);
	
	for (l = stylesheets; l ; l = l->next)
	{
		EmbedStyleSheet *style = (EmbedStyleSheet*) (l->data);
		action = create_stylesheet_action (menu, style);

		g_string_append (xml, "<menuitem name=\"");
		g_string_append (xml, action->name);
		g_string_append (xml, "Item\" verb=\"");
		g_string_append (xml, action->name);
		g_string_append (xml, "\"/>\n");

		/* Make sure all widgets are in the same radio_gruop */
		galeon_radio_action_set_group (GALEON_RADIO_ACTION (action), radio_group);
		radio_group = galeon_radio_action_get_group (GALEON_RADIO_ACTION (action));

		galeon_toggle_action_set_active (GALEON_TOGGLE_ACTION (action), 
						 galeon_embed_stylesheet_compare (style, current));
	}

	g_string_append (xml, "</submenu></placeholder></submenu></menu></Root>");

	/* Don't need to free the contents, they are held as week ref's
	 * against their action */
	g_list_free (stylesheets);
	galeon_embed_stylesheet_free (current);

	p->block_signals = FALSE;

	p->merge_id = egg_menu_merge_add_ui_from_string (p->merge, xml->str, -1, NULL);
	egg_menu_merge_ensure_update (p->merge);
	g_string_free (xml, TRUE);
}

static gboolean
galeon_css_menu_rebuild_timeout_cb (gpointer data)
{
	GaleonCssMenu *gm = data;
	GaleonCssMenuPrivate *p = gm->priv;
	LOG ("GaleonCssMenu rebuild timeout");

	p->rebuild_timeout = 0;

	galeon_css_menu_rebuild (data);
	return FALSE;
}

static void 
galeon_css_menu_rebuild_with_timeout (GaleonCssMenu *gm) 
{
	GaleonCssMenuPrivate *p = gm->priv;

	LOG ("galeon_css_menu_rebuild_with_timeout");

	if (p->rebuild_timeout)
	{
		g_source_remove (p->rebuild_timeout);
	}
	
	p->rebuild_timeout = g_timeout_add (REBUILD_TIMEOUT, 
					    galeon_css_menu_rebuild_timeout_cb, gm);
}


static void
galeon_css_menu_set_embed (GaleonCssMenu *menu, GaleonEmbed *embed)
{
	GaleonCssMenuPrivate *p = menu->priv;

	if (p->embed)
	{
		g_signal_handlers_disconnect_matched (p->embed, G_SIGNAL_MATCH_DATA, 
						      0, 0, NULL, NULL, menu);
		g_object_unref (p->embed);
	}

	p->embed = embed;
	if (p->embed)
	{
		g_object_ref (p->embed);
		g_signal_connect_object (p->embed, "ge_net_stop",
					 G_CALLBACK (galeon_css_menu_rebuild_with_timeout),
					 menu, G_CONNECT_SWAPPED);
	}

	galeon_css_menu_rebuild_with_timeout (menu);
}


static void
embed_changed_cb (GaleonWindow *window, GaleonEmbed *old_embed, 
		  GaleonEmbed *new_embed, GaleonCssMenu *menu)
{
	galeon_css_menu_set_embed (menu, new_embed);
}


static EggActionGroupEntry entries[] = {
	{ "StyleMenuAction", N_("St_yles"),
	  NULL, NULL, N_("Select a different style for this page"),
	  NULL, NULL }
};
static guint n_entries = G_N_ELEMENTS(entries);

static void
galeon_css_menu_set_window (GaleonCssMenu *menu, GaleonWindow *window)
{
	GaleonCssMenuPrivate *p = menu->priv;
	GaleonEmbed *embed;

	p->window = window;
	
	/* Create the Action Group */
	p->merge = g_object_ref (EGG_MENU_MERGE (window->merge));
	menu->priv->menu_action_group = egg_action_group_new ("CssMenuActions");
	egg_menu_merge_insert_action_group(p->merge, p->menu_action_group, 0);
	galeon_action_group_add_actions(p->menu_action_group, entries,
					n_entries, window);

	/* Attach the signals to the window */
	g_signal_connect_object (window, "active-embed-changed",
				 G_CALLBACK (embed_changed_cb),
				 menu, 0);
	embed = galeon_window_get_active_embed (window);

	galeon_css_menu_set_embed (menu, embed);
}


static void
galeon_css_menu_set_property (GObject *object,
			      guint prop_id,
			      const GValue *value,
			      GParamSpec *pspec)
{
	GaleonCssMenu *m = GALEON_CSS_MENU (object);

	switch (prop_id)
	{
	case PROP_WINDOW:
		{
			galeon_css_menu_set_window 
				(m, GALEON_WINDOW (g_value_get_object (value)));
		}
		break;
	default: 
		break;
	}
}

static void
galeon_css_menu_get_property (GObject *object,
			      guint prop_id,
			      GValue *value,
			      GParamSpec *pspec)
{
	GaleonCssMenu *m = GALEON_CSS_MENU (object);
	
	switch (prop_id)
	{
	case PROP_WINDOW:
		g_value_set_object (value, m->priv->window);
		break;
	default: 
		break;
	}
}


static void
galeon_css_menu_finalize_impl(GObject *o)
{
	GaleonCssMenu *gm = GALEON_CSS_MENU(o);
	GaleonCssMenuPrivate *p = gm->priv;

	LOG ("galeon_css_menu_finalize_impl");

	if (p->merge_id > 0)
	{
		egg_menu_merge_remove_ui (p->merge, p->merge_id);
	}

	if (p->menu_action_group)
	{
		egg_menu_merge_remove_action_group
			(p->merge, p->menu_action_group);
		g_object_unref (p->menu_action_group);
	}

	if (p->item_action_group)
	{
		egg_menu_merge_remove_action_group
			(p->merge, p->item_action_group);
		g_object_unref (p->item_action_group);
	}

	if (p->rebuild_timeout)
	{
		g_source_remove (p->rebuild_timeout);
	}

	if (p->embed)
	{
		g_object_unref (p->embed);
	}

	egg_menu_merge_ensure_update (p->merge);
	g_object_unref (p->merge);
	g_free(p);

	G_OBJECT_CLASS(g_object_class)->finalize (o);
}

static void 
galeon_css_menu_init(GaleonCssMenu *m)
{
	GaleonCssMenuPrivate *p = g_new0 (GaleonCssMenuPrivate, 1);
	m->priv = p;
}

static void
galeon_css_menu_class_init(GaleonCssMenuClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	g_object_class = g_type_class_peek_parent(klass);

	object_class->set_property = galeon_css_menu_set_property;
	object_class->get_property = galeon_css_menu_get_property;
	object_class->finalize = galeon_css_menu_finalize_impl;

	g_object_class_install_property (object_class,
                                         PROP_WINDOW,
                                         g_param_spec_object ("GaleonWindow",
                                                              "GaleonWindow",
                                                              "Parent window",
                                                              GALEON_WINDOW_TYPE,
                                                              G_PARAM_READWRITE |
							      G_PARAM_CONSTRUCT_ONLY));
}


MAKE_GET_TYPE(galeon_css_menu, "GaleonCssMenu", GaleonCssMenu,
	      galeon_css_menu_class_init,
	      galeon_css_menu_init, G_TYPE_OBJECT);


GaleonCssMenu *
galeon_css_menu_new (GaleonWindow *window)
{
	GaleonCssMenu *ret = g_object_new (GALEON_TYPE_CSS_MENU, 
					   "GaleonWindow", window, NULL);
	return ret;
}
